@@ -232,9 +232,8 @@ func getMetricNode(oid string, node *Node, nameToNode map[string]*Node) (*Node,
 		_, ok = metricType(n.Type)
 		if ok && metricAccess(n.Access) && len(n.Indexes) == 0 {
 			return n, oidScalar
-		} else {
-			return n, oidSubtree
 		}
+		return n, oidSubtree
 	}
 
 	// Unknown OID/name, search Node tree for longest match.
@@ -252,6 +251,26 @@ func getMetricNode(oid string, node *Node, nameToNode map[string]*Node) (*Node,
 	return n, oidInstance
 }
 
+// In the case of multiple nodes with the same label try to return the node
+// where the OID matches in every branch apart from the last one.
+func getIndexNode(lookup string, nameToNode map[string]*Node, metricOid string) *Node {
+	for _, node := range nameToNode {
+		if node.Label != lookup {
+			continue
+		}
+
+		oid := strings.Split(metricOid, ".")
+		oidPrefix := strings.Join(oid[:len(oid)-1], ".")
+
+		if strings.HasPrefix(node.Oid, oidPrefix) {
+			return node
+		}
+	}
+
+	// If no node matches, revert to previous behavior.
+	return nameToNode[lookup]
+}
+
 func generateConfigModule(cfg *ModuleConfig, node *Node, nameToNode map[string]*Node, logger log.Logger) (*config.Module, error) {
 	out := &config.Module{}
 	needToWalk := map[string]struct{}{}
@@ -341,7 +360,10 @@ func generateConfigModule(cfg *ModuleConfig, node *Node, nameToNode map[string]*
 				return // Ignored metric.
 			}
 
+			// Afi (Address family)
 			prevType := ""
+			// Safi (Subsequent address family, e.g. Multicast/Unicast)
+			prev2Type := ""
 			for count, i := range n.Indexes {
 				index := &config.Index{Labelname: i}
 				indexNode, ok := nameToNode[i]
@@ -364,11 +386,14 @@ func generateConfigModule(cfg *ModuleConfig, node *Node, nameToNode map[string]*
 				if subtype, ok := combinedTypes[index.Type]; ok {
 					if prevType == subtype {
 						metric.Indexes = metric.Indexes[:len(metric.Indexes)-1]
+					} else if prev2Type == subtype {
+						metric.Indexes = metric.Indexes[:len(metric.Indexes)-2]
 					} else {
 						level.Warn(logger).Log("msg", "Can't handle index type on node, missing preceding", "node", n.Label, "type", index.Type, "missing", subtype)
 						return
 					}
 				}
+				prev2Type = prevType
 				prevType = indexNode.TextualConvention
 				metric.Indexes = append(metric.Indexes, index)
 			}
@@ -413,7 +438,7 @@ func generateConfigModule(cfg *ModuleConfig, node *Node, nameToNode map[string]*
 				if _, ok := nameToNode[lookup.Lookup]; !ok {
 					return nil, fmt.Errorf("unknown index '%s'", lookup.Lookup)
 				}
-				indexNode := nameToNode[lookup.Lookup]
+				indexNode := getIndexNode(lookup.Lookup, nameToNode, metric.Oid)
 				typ, ok := metricType(indexNode.Type)
 				if !ok {
 					return nil, fmt.Errorf("unknown index type %s for %s", indexNode.Type, lookup.Lookup)
@@ -503,6 +528,8 @@ func generateConfigModule(cfg *ModuleConfig, node *Node, nameToNode map[string]*
 		for _, metric := range out.Metrics {
 			if name == metric.Name || name == metric.Oid {
 				metric.RegexpExtracts = params.RegexpExtracts
+				metric.Offset = params.Offset
+				metric.Scale = params.Scale
 			}
 		}
 	}
